<?php
declare(strict_types=1);

namespace Ep\TecDoc\Middleware;

use Ep\TecDoc\TecDocService;
use Laminas\Diactoros\Response\HtmlResponse;
use Psr\Container\ContainerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

final class LinkedVehicles implements MiddlewareInterface
{
    private TecDocService $tecdoc;

    public function __construct(ContainerInterface $container)
    {
        $this->tecdoc = $container->get(TecDocService::class);
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $articleId = \intval(ltrim($request->getUri()->getPath(), '/'));

        return new HtmlResponse(
            $this->render($articleId)
        );
    }

    private function render(int $articleId): string
    {
        $vehicles = $this->tecdoc->getVehicleStringsLinkedToArticle($articleId);

        if (count($vehicles) > 0):
            ob_start();
            ?>
            <h3 class="accordion_title"><?= __('Povezana Vozila', 'ep-tecdoc'); ?></h3>

            <div class="accordion" rel="1">
                <?php foreach ($vehicles as $brand => $_vehicles): ?>
                    <div class="accordion-item">
                        <a href="#" class="accordion-title plain">
                            <button class="toggle"><i class="icon-angle-down"></i></button>
                            <span><?= $brand; ?></span>
                        </a>

                        <div class="accordion-inner">
                            <ul>
                                <?php foreach ($_vehicles as $vehicle): ?>
                                    <li>
                                        <?php
                                        $url = preg_replace('#' . WP_HOME . '#', '', get_permalink(wc_get_page_id('shop')));
                                        $url = WP_HOME . '/' . $vehicle['slug'] . $url;

                                        printf('<a href="%s">%s</a>', $url, $vehicle['label']);
                                        ?>
                                    </li>
                                <?php endforeach; ?>
                            </ul>
                        </div>
                    </div>
                <?php endforeach; ?>
            </div><?php
            return ob_get_clean();
        endif;
        
        return '';
    }
}