<?php
namespace Ep\TecDoc\Plugin;

use Ak\Wp\Plugin\PluginInterface;
use Ep\TecDoc\TecDocService;
use WP_Query;
use WP_Term;

/**
 * Class TecDocCategory
 *
 * @package Ep\TecDoc\Plugin
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/tecdoc
 */
class TecDocCategory implements PluginInterface
{
    private TecDocService $tecdoc;

    /**
     * TecDocCategory constructor.
     * @param TecDocService $tecdoc
     */
    public function __construct(TecDocService $tecdoc)
    {
        $this->tecdoc = $tecdoc;
    }

    public function attach(): void
    {
        add_filter('posts_pre_query', [$this, 'posts_pre_query'], 5, 2);
        add_filter('term_link', [$this, 'term_link'], 10, 3);
    }

    public function posts_pre_query($null, WP_Query $query)
    {
        $category = get_queried_object();

        if (!$query->is_main_query() || !$category instanceof WP_Term || $category->taxonomy !== 'product_cat') {
            return;
        }

        if ($genericArticleIds = get_term_meta($category->term_id, 'genericArticleIds', true)) {
            $genericArticleIds = array_filter(array_map('absint', explode(';', $genericArticleIds)));
            $articles = $this->tecdoc->getArticlesByGenericIds($genericArticleIds);

            $articleKeys = array_map(function ($article) {
                return $article['key'];
            }, $articles);

            if (count($articles) > 0) {
                $query->tecdocArticles = $articles;
                $query->set('tecdoc_keys', $articleKeys);
            } else {
                $query->set('ID', 0);
            }

            return $null;
        }
    }

    public function term_link($url, /** @noinspection PhpUnusedParameterInspection */$term, $taxonomy)
    {
        if ($taxonomy === 'product_cat' && ($vehicle = $this->tecdoc->getSelectedVehicle())) {
            $homeUrl = rtrim(home_url(), '/');

            if (empty($homeUrl)) {
                $url = '/' . $vehicle['slug'] . '/' . ltrim($url, '/');
            } else {
                $url = str_replace($homeUrl, $homeUrl . '/' . $vehicle['slug'], $url);
            }
        }

        return $url;
    }
}