<?php
declare(strict_types=1);

namespace Ep\TecDoc\Plugin;

use Ak\Wp\Plugin\PluginInterface;
use Ep\Gath\ImageManager;
use Ep\Gath\Plugin\Products;
use Ep\TecDoc\TecDocService;

/**
 * Class TecDocImages
 *
 * @package Ep\TecDoc\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/ep-tecdoc
 */
final class TecDocImages implements PluginInterface
{
    private const IGNORE_BRANDS = [
        826, 827, 828, 829, 830, 831, 832, 833, 834, 835, 836, 837, 838,
        839, 840, 841, 842, 843, 844, 845
    ];

    private const VALID_META_KEYS = [
        '_product_image_gallery',
        '_thumbnail_id',
    ];
    
    private ImageManager $images;

    /**
     * TecDocCategory constructor.
     * @param ImageManager $images
     */
    public function __construct(ImageManager $images)
    {
        $this->images = $images;
    }

    public function attach(): void
    {
        add_filter('get_post_metadata', [$this, 'get_post_metadata'], 10, 3);
    }

    public function get_post_metadata($null, $object_id, $meta_key)
    {
        if ($object_id < Products::ID_OFFSET || !in_array($meta_key, self::VALID_META_KEYS)) {
            return $null;
        }

        $brands = get_the_terms($postId, 'product_brand');
        $brand = is_array($brands) && count($brands) > 0 ? $brands[0] : null;
        
        if ($brand instanceof \WP_Term && in_array(intval($brand->term_id - Products::TERM_OFFSET_BRAND), self::IGNORE_BRANDS)) {
            return $null;
        }

        global $wp_query;
        $tecdocKey = get_post_meta($object_id, '_tecdoc_key', true);
        $tecdocArticle = null;
        
        if (isset($wp_query->tecdocArticles) && is_array($wp_query->tecdocArticles)) {
            foreach ($wp_query->tecdocArticles as $article) {
                if ($article['key'] === $tecdocKey) {
                    $tecdocArticle = $article;
                    break;
                }
            }
        }

        if ($tecdocArticle === null || !isset($tecdocArticle['images']) || !is_array($tecdocArticle['images']) || empty($tecdocArticle['images'])) {
            return $null;
        }

        switch ($meta_key) {
            case '_thumbnail_id':
                return $this->images->getImageId($tecdocArticle['images'][0]);
                break;
            case '_product_image_gallery':
                return implode(',', array_map(
                    [$this->images, 'getImageId'],
                    array_slice($tecdocArticle['images'], 1)
                ));
                break;
        }

        return $null;
    }
}