<?php
declare(strict_types=1);

namespace Ep\TecDoc\Plugin;

use Ak\Wp\Plugin\PluginInterface;
use Ep\TecDoc\TecDocService;

/**
 * Class TecDocSearch
 *
 * @package Ep\TecDoc\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/ep-tecdoc
 */
final class TecDocSearch implements PluginInterface
{
    private TecDocService $tecdoc;

    /**
     * @param TecDocService $tecdoc
     */
    public function __construct(TecDocService $tecdoc)
    {
        $this->tecdoc = $tecdoc;
    }

    public function attach(): void
    {
        add_action('pre_get_posts', [$this, 'pre_get_posts'], 0);
        add_filter('the_posts', [$this, 'post_get_posts'], 10, 2);
    }

    /**
     * @param \WP_Query $query
     * @return void
     */
    public function pre_get_posts(\WP_Query $query)
    {
        if (!is_admin() && $query->is_main_query() && $query->is_search) {
            $searchQuery = get_search_query();
            $query->set('post_type', 'product');

            if (strlen($searchQuery) >= 4) {
                $articles = $this->tecdoc->searchArticlesByNumber(
                    $searchQuery,
                    null, null, 1, null, null,
                    true
                );
                $query->searchedArticles = $articles;

                if (count($articles) > 0) {
                    $articleKeys = array_map(function($article) {
                        return sprintf('%d|%s', $article['brandNo'], $article['articleNo']);
                    }, $articles);

                    $query->set('tecdoc_keys', $articleKeys);
                }
            }
        }
    }

    public function post_get_posts(array $posts, \WP_Query $query)
    {
        $articleKeys = array_filter(
            array_map(
                fn (\WP_Post $post) => get_post_meta($post->ID, '_tecdoc_key', true) ?: false,
                $posts
            )
        );

        if (isset($query->tecdocArticles)) {
            $loadedKeys = array_map(
                fn (array $article) => $article['key'],
                $query->tecdocArticles
            );

            $articleKeys = array_diff($articleKeys, $loadedKeys);
        }

        if (!empty($articleKeys)) {
            if (isset($query->searchedArticles) && is_array($query->searchedArticles)) {
                $articleIds = [];

                foreach ($query->searchedArticles as $article) {
                    $articleKey = sprintf('%d|%s', $article['brandNo'], $article['articleNo']);
                    if (in_array($articleKey, $articleKeys)) {
                        $articleIds[] = $article['articleId'];
                    }
                }
            } else {
                $articleIds = \array_map(
                    [$this->tecdoc, 'getArticleId'],
                    $articleKeys
                );
            }

            $articleIds = array_filter($articleIds);

            if (!empty($articleIds)) {
                $query->tecdocArticles = array_merge(
                    $query->tecdocArticles ?? [],
                    $this->tecdoc->getArticlesByIds($articleIds)
                );
            }
        }

        return $posts;
    }
}